program BoxCast_P2J;

{
  Pascal version of the original BoxCast example Web app client from here:

   https://macpgmr.github.io/MacXPlatform/PascalDynLibs_3.html#Mapbox

  Author:    Phil Hess.
  Copyright: Copyright (C) Phil Hess.
  License:   Modified LGPL (see Free Pascal's rtl/COPYING.FPC).
}

uses
  JS,
  Web,
  CorsliteFunc,
  MapboxGL,
  BoxCast_Config;


type
   {Custom map control for About button}
  TAboutButtonControl = class  //class(IControl)  //IControl is not really a class
  private
    _map       : TMap;
    _container : TJSHTMLElement;
  public
    constructor New;  //added this for now
    function onAdd(map : TMap) : TJSHTMLELement; //override;
    procedure onRemove(map : TMap = nil); //override;
  end;

constructor TAboutButtonControl.New;
begin
end;

function TAboutButtonControl.onAdd(map : TMap) : TJSHTMLElement;
begin
  _map := map;
  _container := TJSHTMLElement(document.createElement('button'));
  _container.className := 'mapboxgl-ctrl';  {Use Mapbox styling}
  _container.innerHTML := '<b>About</b>';
  Result := _container;
end;

procedure TAboutButtonControl.onRemove(map : TMap);
begin
  _container.parentNode.removeChild(_container);
  _map := TMap(Undefined);
end;


type
   {GeoJSON data returned by server call}
  TFcstGeometry = class
    &type       : string;
    coordinates : array of Double;  //array [0..1] of Double;
    end;

  TFcstProperties = class
    high : string;
    low  : string;
    end;

  TFcstFeature = class
    &type      : string;
    geometry   : TFcstGeometry;
    properties : TFcstProperties;
  end;

  TFcstGeoJSON = class(TJSObject)
    &type    : string;
    features : array of TFcstFeature;
  end;


var
  map          : TMap;    {The map}
  aboutPopup   : TPopup;  {About box}
  resultsPopup : TPopup;  {Results box}
  curCursor    : string = 'pointer';  {Current cursor}


function AboutOnClick(e : TJSMouseEvent) : Boolean;
 {Called when About button is clicked.}
begin
  aboutPopup := TPopup.New(new(['anchor', 'bottom']));
  aboutPopup.setLngLat(map.getCenter());
  aboutPopup.setHTML(
              '<center><h3>' + appSettings.name + ', version ' + appSettings.version + '</h3>' +
              '<h4>Powered by <a href="https://mapbox.com" target="_blank">Mapbox GL JS</a>, version ' + 
              version + '<br>' +
              'Weather forecasts from <a href="http://www.usgovxml.com/DataService.aspx?ds=NDFD" ' +
              'target="_blank">National Digital Forecast Database</a></h4>' +
              'Click anywhere in the U.S. to see today''s weather forecast.<p>' +
              '<button onclick="pas.program.CloseAbout()"><b>Close</b></button></center>');
  aboutPopup.addTo(map);
end;


function CloseAbout : Boolean;
 {Close the About box.}
begin
  aboutPopup.remove;
end;


procedure MapOnClick(e : TMapEvent);
 {Called when map is clicked.}

   //using nested function so can access e (above)
  procedure CorsCallback(err  : TObject; 
                         resp : TJSXMLHttpRequest);
  var
    popupText : string;
    fcst      : TFcstGeoJSON;
  begin
    if Assigned(err) then
      begin
      popupText := 'Error retrieving forecast.';
      console.log(popupText);
      end
    else
      begin
      try
        fcst := TFcstGeoJSON(TJSJSON.parseObject(resp.responseText));
        popupText := '<center>Today''s forecast<br>' +
                     'High: <b>' + fcst.features[0].properties.high + '</b><br>' +
                     'Low: <b>' + fcst.features[0].properties.low + '</b><p>' +
                     '<button onclick="pas.program.CloseResults()"><b>Close</b></button>' +
                     '</center>';
      except
        popupText := 'Forecast data returned is invalid.';
        end;
      end;
    curCursor := 'pointer';
    map.getCanvas().style.setProperty('cursor', curCursor);
    resultsPopup := TPopup.New();
    with resultsPopup do
      begin
      setLngLat(e.lngLat);
      setHTML(popupText);
      addTo(map);
      end;
  end;

begin
  curCursor := 'wait';
  map.getCanvas().style.setProperty('cursor', curCursor);

   {Make AJAX call to server, passing coordinates of clicked point.
     corslite will call the callback function with server response.}
  corslite(appSettings.fcstUrl + '?lat1=' + string(JSValue(e.lngLat.lat)) + 
            '&lon1=' + string(JSValue(e.lngLat.lng)) + '&geojson=true',
           @CorsCallback, True);
end;


function CloseResults : Boolean;
 {Close the Results box.}
begin
  resultsPopup.remove;
end;


procedure MapOnMouseEnter(e : TMapEvent);
 {Called when mouse is over clickable area.
   Change mouse style to pointer or wait.}
begin
  map.getCanvas().style.setProperty('cursor', curCursor);
end;


procedure MapOnMouseLeave(e : TMapEvent);
 {Called when mouse is over non-clickable area.
   Change mouse style back to default.}
begin
  map.getCanvas().style.setProperty('cursor', '');
end;


procedure MapOnLoad(e : TMapEvent);
 {Set up map. Called when the map loads.}
var
  aboutButton : TAboutButtonControl;
begin
   {Add navigation control to map}
  map.addControl(IControl(TNavigationControl.New()), 'top-left');

   {Add scale control to map}
  map.addControl(IControl(TScaleControl.New(new(['maxWidth', 100,
                                                 'unit', 'imperial']))), 
                 'bottom-left');

   {Add About button to map}
  aboutButton := TAboutButtonControl.New;  //New();
  map.addControl(IControl(aboutButton));
  aboutButton._container.onclick := @AboutOnClick;

   {Add fullscreen control to map}
  map.addControl(IControl(TFullscreenControl.New()));

   {Add layer with bounding box of "lower 48" U.S. states for detecting click}
  map.addLayer(
   new(['id', 'click-box',
        'type', 'fill',
        'source', new([
          'type', 'geojson',
          'data', new([
            'type', 'Feature',
            'geometry', new([
              'type', 'Polygon',
              'coordinates', TJSArray._of(
                               TJSArray._of(
                                 TJSArray._of(-124.848974, 24.396308),
                                 TJSArray._of(-124.848974, 49.384358),
                                 TJSArray._of( -66.885444, 49.384358),
                                 TJSArray._of( -66.885444, 24.396308),
                                 TJSArray._of(-124.848974, 24.396308)))
             ])
           ])
         ]),
        'layout', new([]),
        'paint', new(['fill-opacity', 0])
       ]));
(*
   //another way to do it is to specify as JSON and parse it:
  map.addLayer(TJSJSON.parseObject(
   '{' +
     '"id": "click-box",' +
     '"type": "fill",' +
     '"source": {' +
       '"type": "geojson",' +
       '"data": {' +
         '"type": "Feature",' +
         '"geometry": {' +
           '"type": "Polygon",' +
           '"coordinates": [[[-124.848974, 24.396308],' +
                            '[-124.848974, 49.384358],' +
                            '[ -66.885444, 49.384358],' +
                            '[ -66.885444, 24.396308],' +
                            '[-124.848974, 24.396308]]]' +
         '}' +
       '}' +
     '},' +
     '"layout": {},' +
     '"paint": {' +
       '"fill-opacity": 0' +
     '}' +
   '}'));
*)

   {Add click handler}
  map.on('click', 'click-box', @MapOnClick);

   {Add handler for when mouse is over clickable area}
  map.on('mouseenter', 'click-box', @MapOnMouseEnter);

   {Add handler for when mouse is over non-clickable area}
  map.on('mouseleave', 'click-box', @MapOnMouseLeave);
end;


begin
  accessToken := appsettings.accessToken;

   {Create and initialize map}
  map := TMap.New(new(['container', 'map',
                       'style', 'mapbox://styles/mapbox/satellite-streets-v10',
                       'center', TJSArray._of(-107.74, 37.92), 
                       'zoom', 9.0,
                       'logoPosition', 'bottom-right']));

   {Add handler for when map loads}
  map.on('load', @MapOnLoad);
end.
